//////////////////////////////////////////////////////////////////////////////////
// 
// DSairLite ParamManager
// Copyright(C)'2023- DesktopStation
// 
// 本ソフトウェアは、上記の著作者によって開発されたものであり、利用には
// ライセンス契約が必要となります。
// 
//////////////////////////////////////////////////////////////////////////////////
//

#include <arduino.h>
#include <EEPROM.h>
#include "ParamManager.h"
#include "DSVersion.h"


//初期化する
ParamManager::ParamManager()
{
	

	memset(u4_data, 0, sizeof(u4_data));

	Clear();





}

void ParamManager::Clear()
{
	//BIDI CUTOFF
	u4_FlagFunction = 0;//PARAMFLAG_OUTBIDICUTOUT;

	for( int i = 0; i < PARAMSIZE_DATA; i++)
	{
		u4_data[i] = 0;
	}

	//初期値
	u4_data[0] = 0x00;//CRC
	u4_data[PARAMTYPE_FIRMWARE] = FIRMWARE_VERSION;
	u4_data[PARAMTYPE_CUR_LOCOADDR] = 3;

	u4_data[PARAMTYPE_MOMENTARY] = 0b100;
	u4_data[PARAMTYPE_LOGADDRRANGE] = 0;


}

void ParamManager::FactoryReset()
{

	EEPROM.begin(1024);
	
	for( int i = 0; i < PARAMSIZE_DATA; i++)
	{
		EEPROM.write(i * 4 + 0, 0x00);
		EEPROM.write(i * 4 + 1, 0x00);
		EEPROM.write(i * 4 + 2, 0x00);
		EEPROM.write(i * 4 + 3, 0x00);
	}

	EEPROM.commit();

	EEPROM.end();

}

void ParamManager::Save()
{

	EEPROM.begin(1024);

	for( int i = 0; i < PARAMSIZE_DATA; i++)
	{
		EEPROM.write(i * 4 + 0, u4_data[i] & 0xFF);
		EEPROM.write(i * 4 + 1, (u4_data[i] >> 8) & 0xFF);
		EEPROM.write(i * 4 + 2, (u4_data[i] >> 16) & 0xFF);
		EEPROM.write(i * 4 + 3, (u4_data[i] >> 24) & 0xFF);
	}

	if( EEPROM.commit())
	{
		//Serial.println("WRITE OK");
	}
	else
	{
		//Serial.println("WRITE FAILED");		
	}

	EEPROM.end();

}

void ParamManager::Load()
{

	EEPROM.begin(1024);

	for( int i = 0; i < PARAMSIZE_DATA; i++)
	{
		u4_data[i] = (uint32_t)EEPROM.read(i * 4 + 0) + ((uint32_t)EEPROM.read(i * 4 + 1) << 8) + ((uint32_t)EEPROM.read(i * 4 + 2) << 16) + ((uint32_t)EEPROM.read(i * 4 + 3) << 24);
	}

	EEPROM.end();

	//チェック
	if( (u4_data[PARAMTYPE_VER] == 0xFFFFFFFF) && (u4_data[PARAMTYPE_FIRMWARE] == 0xFFFFFFFF) )
	{
		//値が一致しない
		Clear();
	} 

/*
	Serial.println(u4_data[PARAMTYPE_VER]);
	Serial.println(u4_data[PARAMTYPE_FIRMWARE]);
	Serial.println(u4_data[PARAMTYPE_CUR_LOCOADDR]);
*/

}


uint32_t ParamManager::GetParameter(uint32_t u4_index)
{
	uint32_t u4_temp_data = 0;

	if( u4_index >= PARAMSIZE_DATA)
	{
		//範囲外

	}
	else
	{
		u4_temp_data = u4_data[u4_index];

		switch(u4_index)
		{

			case PARAMTYPE_CRC:
				break;

			case PARAMTYPE_VER:
				break;

			case PARAMTYPE_FIRMWARE	:
				break;

			case PARAMTYPE_COUNT_UPDATE:
				break;

			case PARAMTYPE_CUR_LOCOADDR:
				break;

			case PARAMTYPE_CUR_ACCADDR:
				break;

			case PARAMTYPE_CUR_FUNCTION:
				break;

			case PARAMTYPE_CUR_DIRECTION:
				break;

			case PARAMTYPE_MOMENTARY:
				break;

			case PARAMTYPE_BIDICOUT:
				break;

			case PARAMTYPE_FUNCTION:
				break;

			case PARAMTYPE_OCLEVEL:
				break;

			case PARAMTYPE_OVLEVEL:
				break;

			case PARAMTYPE_LVLEVEL:
				break;

		}
	}



	return u4_temp_data;

}

void ParamManager::SetParamerter(uint32_t u4_index, uint32_t u4_param_data)
{

	if( u4_index >= PARAMSIZE_DATA)
	{
		//範囲外

	}
	else
	{

		u4_data[u4_index] = u4_param_data;
	}

}


bool ParamManager::GetFlagBidiCutout()
{
	bool aRet = false;

	if( (u4_FlagFunction & PARAMFLAG_OUTBIDICUTOUT) > 0 )
	{
		aRet = true;
	}

	return aRet;

}
